/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "Apache BSF", "Apache", and "Apache Software Foundation"
 *    must not be used to endorse or promote products derived from
 *    this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many individuals
 * on behalf of the Apache Software Foundation and was originally created by
 * Sanjiva Weerawarana and others at International Business Machines
 * Corporation. For more information on the Apache Software Foundation,
 * please see <http://www.apache.org/>.
 */

package org.apache.bsf.debug.util;

import java.rmi.RemoteException;
import org.apache.bsf.debug.util.DebugConstants;

public class Skeleton 
{
	
	private static int gUidGenerator=DebugConstants.FIRST_NON_RESERVED_UID;
	private static Object lock = new Object();
	
	int m_tid,m_uid;
	SocketConnection m_con;
	
	//---------------------------------------------
	protected Skeleton(int tid) throws RemoteException {
		m_tid = tid;
		m_uid = DebugConstants.UNKNOWN_UID;
	}
	//---------------------------------------------
	protected Skeleton(int tid,int uid) throws RemoteException  {
		m_tid = tid;
		m_uid = uid;
	}
	//---------------------------------------------
	public boolean equals(Object o) {
		if (o instanceof Skeleton)
			return m_uid == ((Skeleton)o).m_uid;
		else 
			return false;
	}
	//---------------------------------------------
	public void allocOid(SocketConnection con) {
		synchronized(lock) {
			m_con = con;
			if (m_uid==DebugConstants.UNKNOWN_UID) {
				m_uid = gUidGenerator++;
			}
		}
	}
	//---------------------------------------------
	public boolean hasNoUid() {
		return m_uid==DebugConstants.UNKNOWN_UID;
	}
	//---------------------------------------------
	public int getTid() { return m_tid; }
	public int getUid() { return m_uid; }
	
	public void addListener(RemoteServiceListener l) throws RemoteException {
		throw new Error("Not to be called on the local service.");
	}
	public void removeListener(RemoteServiceListener l) throws RemoteException {
		throw new Error("Not to be called on the local service.");
	}		
	public void createFuture(Object requester) throws RemoteException  {
		throw new Error("Not to be called on the local service.");
	}		
	public void suspendFuture(Object requester) throws RemoteException  {
		throw new Error("Not to be called on the local service.");
	}
	public void completeFuture(Object requester) throws RemoteException  {
		throw new Error("Not to be called on the local service.");
	}

}

